import { OnDestroy, OnChanges, OnInit, EventEmitter, ElementRef, SimpleChanges } from '@angular/core';
import * as chartJs from 'chart.js';
import { Color } from './color';
export declare type SingleDataSet = (number[] | chartJs.ChartPoint[]);
export declare type MultiDataSet = (number[] | chartJs.ChartPoint[])[];
export declare type SingleOrMultiDataSet = SingleDataSet | MultiDataSet;
export interface PluginServiceGlobalRegistration {
    id?: string;
}
export interface PluginServiceRegistrationOptions {
    beforeInit?(chartInstance: Chart, options?: any): void;
    afterInit?(chartInstance: Chart, options?: any): void;
    beforeUpdate?(chartInstance: Chart, options?: any): void;
    afterUpdate?(chartInstance: Chart, options?: any): void;
    beforeLayout?(chartInstance: Chart, options?: any): void;
    afterLayout?(chartInstance: Chart, options?: any): void;
    beforeDatasetsUpdate?(chartInstance: Chart, options?: any): void;
    afterDatasetsUpdate?(chartInstance: Chart, options?: any): void;
    beforeDatasetUpdate?(chartInstance: Chart, options?: any): void;
    afterDatasetUpdate?(chartInstance: Chart, options?: any): void;
    beforeRender?(chartInstance: Chart, options?: any): void;
    afterRender?(chartInstance: Chart, options?: any): void;
    beforeDraw?(chartInstance: Chart, easing: string, options?: any): void;
    afterDraw?(chartInstance: Chart, easing: string, options?: any): void;
    beforeDatasetsDraw?(chartInstance: Chart, easing: string, options?: any): void;
    afterDatasetsDraw?(chartInstance: Chart, easing: string, options?: any): void;
    beforeDatasetDraw?(chartInstance: Chart, easing: string, options?: any): void;
    afterDatasetDraw?(chartInstance: Chart, easing: string, options?: any): void;
    beforeTooltipDraw?(chartInstance: Chart, tooltipData?: any, options?: any): void;
    afterTooltipDraw?(chartInstance: Chart, tooltipData?: any, options?: any): void;
    beforeEvent?(chartInstance: Chart, event: Event, options?: any): void;
    afterEvent?(chartInstance: Chart, event: Event, options?: any): void;
    resize?(chartInstance: Chart, newChartSize: Chart.ChartSize, options?: any): void;
    destroy?(chartInstance: Chart): void;
    /** @deprecated since version 2.5.0. Use `afterLayout` instead. */
    afterScaleUpdate?(chartInstance: Chart, options?: any): void;
}
export declare type PluginServiceGlobalRegistrationAndOptions = PluginServiceGlobalRegistration & PluginServiceRegistrationOptions;
export declare class BaseChartDirective implements OnDestroy, OnChanges, OnInit {
    private element;
    data: SingleOrMultiDataSet;
    datasets: chartJs.ChartDataSets[];
    labels: string[];
    options: chartJs.ChartOptions;
    chartType: chartJs.ChartType;
    colors: Color[];
    legend: boolean;
    chartClick: EventEmitter<{
        event?: MouseEvent;
        active?: {}[];
    }>;
    chartHover: EventEmitter<{
        event: MouseEvent;
        active: {}[];
    }>;
    ctx: string;
    chart: Chart;
    private initFlag;
    /**
     * Register a plugin.
     */
    static registerPlugin(plugin: PluginServiceGlobalRegistrationAndOptions): void;
    constructor(element: ElementRef);
    ngOnInit(): void;
    ngOnChanges(changes: SimpleChanges): void;
    ngOnDestroy(): void;
    getChartBuilder(ctx: string): Chart;
    private isChartDataSetsArray;
    private updateChartData;
    private isSingleDataSet;
    private getDatasets;
    private refresh;
}
